<?php
//*************************************************************************************************
//
//		BASIC FUNCTIONS
//	
//*************************************************************************************************

	function getSearchColumn($column, $search, $delimiter=' ')
	{
		if ( $search != '' )
		{
			$count		= 0;
			$retVal		= '';
			$search		= handleSingleQuote($search);
			$arrNeedle  = explode($delimiter, $search);

			foreach ( $arrNeedle as $needle )
			{
				if ( $count++ > 0 )
					$retVal .= " AND ";

				$retVal .= " ".$column." LIKE '%".$needle."%' ";
			}

			return $retVal;
		}
		else
			return '';
	}

//*************************************************************************************************

	function getHighlightedSearchText($search, $searchText, $delimiter=' ', $prefix='<font color="red">', $suffix='</font>')
	{
		if ( $search != '' )
		{
			$retVal		= strip_tags($search);
			$arrNeedle  = explode($delimiter, strtolower($searchText));

			$prefixLen	= strlen($prefix);
			$suffixLen	= strlen($suffix);

			foreach ( $arrNeedle as $needle )
			{				
				$prevPos	= 0;
				$fPos		= 0;
				$count		= 0;
				
				do
				{
					$count++;
					$searchEx	= strtolower($retVal);
					$searchEx	= str_replace($prefix, str_repeat("~", $prefixLen), $searchEx);
					$searchEx	= str_replace($suffix, str_repeat("~", $suffixLen), $searchEx);
					$fPos		= strpos($searchEx, $needle, $prevPos);

					if ( !($fPos === false) )
					{						
						if ( $prevPos >= $fPos && $fPos != 0 )
						{
							//echo "[$prevPos equal $fPos]<br>";
							break;
						}
						
						$needleLen	= strlen($needle);
						$first		= substr($retVal, 0, $fPos);
						$mid		= substr($retVal, $fPos, $needleLen);
						$last		= substr($retVal, $fPos+$needleLen);
						$retVal		= $first.$prefix.$mid.$suffix.$last;

						$prevPos	= $fPos+$needleLen+$prefixLen+2;
					}
				}
				while ( !($fPos===false) && $count < 5 );
			}

			return $retVal;
		}
		else
			return '';
	}

//*************************************************************************************************

	function getHighlightedSearchTextEx($search, $searchText, $maxlength=150, $delimiter=' ', $prefix='<font color="red">', $suffix='</font>')
	{
		if ( $search != '' )
		{
			$retVal		= '';
			
			$searchEx	= strtolower(strip_tags($search));
			$arrNeedle  = explode($delimiter, strtolower($searchText));
			$partLen	= ceil($maxlength/4);
			
			$fPos		= 0;
			$count		= 0;
			$prevLimit	= 0;

			foreach ( $arrNeedle as $needle )
			{
				if ( $count > 1 ) return $retVal;

				$fPos = strpos($searchEx, $needle, $fPos);
				
				if ( $fPos )
				{
					if ( $fPos > $prevLimit )
					{
						$count++;
						$needleLen	= strlen($needle);
						
						if ( ($fPos-$partLen) < 1 )
							$first = substr($searchEx, 0, $fPos);
						else
							$first = '...'.substr($searchEx, ($fPos-$partLen), $partLen);
						
						$mid		= substr($searchEx, $fPos, $needleLen);
						$last		= substr($searchEx, $fPos+$needleLen, $partLen).'...';
						$retVal		.= $first.$mid.$last;

						$prevLimit	= $fPos+$needleLen+$partLen;
					}
					//echo "[$fPos, $prevLimit]";
				}
			}
			
			// If Detail Length is small, Make it full
			$retValLen = strlen($retVal);
			$retValDiff= $maxlength-$partLen;
			$lenSearch = strlen($searchEx);

			if ( $retValLen < $retValDiff )
			{				
				if ( ($prevLimit+$retValDiff) < $lenSearch ) 
					$retVal .= substr($searchEx, $prevLimit, $retValDiff).'...';

				else
					$retVal = '...'.substr($searchEx, 0, $partLen*2).$retVal;
			}

			$retVal = getHighlightedSearchText($retVal, $searchText);
			return $retVal;
		}
		else
			return '';
	}

//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************

	function getPartialString($str, $max=150, $link='')
	{																				
		$retVal = "";

		if ( strlen($str) > ($max+3) )
		{
			$sPos = strpos($str, " ", $max);

			if ( $sPos )
				$retVal .= substr($str, 0, $sPos )."...";
			else
				$retVal .= substr($str, 0, 150 )."...";

			if ( $link != '' )
				$retVal .= " <a href=\"".$link."\" style='font-size:10px'>".'Readmore'."</a>";
		}
		else
			$retVal .= $str ;

		return $retVal;
	}

//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************

	function validateCategoryName($msg, $title, $filename, $categoryId=0)
	{
		require(PATH."include/constants.php");

		if ( $USE_MOD_REWRITE == 'Y' )
		{
			if ( $filename == "" )
				$filename = $title;

			$filename = trim($filename);
			if ( $filename == "" )
			{
				$msg = "Category name cannot be empty. -validateCategoryName-";
				return 0;
			}

			$filename = replaceFilenameChars($filename);
			$filename = str_replace(" ", "\$", $filename); // Replace Space with $
			
			if ( ereg($CATEGORY_ALLOWED_CHARS, $filename) )
			{
				$msg = "Category name can only contain alphanumeric characters, ('), (.), (-) and (_) -validateCategoryName-";
				return 0;
			}
			$filename = str_replace("\$", "-", $filename); // Replaced back [-]

			if ( strstr($filename, " + ") )
			{
				$msg = "Category name cannot contain + sign with spaces at both ends -validateCategoryName-";					
				return 0;
			}

			//$filename = str_replace("&", "", $filename);					// Remove &
			if ( stristr($filename, "&") && !stristr($filename, "-&-") )
			{
				$msg = "Error in category name. `&` can only be used with spaces around it.<br>Please update title so that & has leading and trailing whitespaces. -validateCategoryName-";					
				return 0;
			}
/*
CATEGORY FILENAME IS INSIGNIFICANT TO ID
			if ( $categoryId > 0 )
				$res = executeQuery("SELECT ID FROM ".PREFIX."CATEGORY WHERE FILENAME = '".$filename."' AND ID <> ".$categoryId."");
			else
				$res = executeQuery("SELECT ID FROM ".PREFIX."CATEGORY WHERE FILENAME = '".$filename."'");

			if ( $row = getRow($res) )
			{
				$msg = "Filename should be unique. It cannot match other category`s filename.";
				return 0;
			}
*/

			return 1;
		}	
		return 1;

	}

//*************************************************************************************************

	function validateProductName($msg, $title, $filename, $categoryId=0, $productId=0)
	{
		require(PATH."include/constants.php");

		if ( $USE_MOD_REWRITE == 'Y' )
		{
			if ( $filename == "" )
				$filename = $title;

			$filename = trim($filename);
			if ( $filename == "" )
			{
				$msg = "Product name cannot be empty. -validateProductName-";
				return 0;
			}

			$filename = replaceFilenameChars($filename);
			$filename = str_replace(" ", "\$", $filename);  // Repalce SPACE with - BUT for timebeing with $ AS [-] fails in ereg

			if ( ereg($PRODUCT_ALLOWED_CHARS, $filename) )
			{
				$msg = "Filename can only contain alphanumeric characters, ('), (.), (-) and (_). -validateProductName-";
				return 0;
			}
			$filename = str_replace("\$", "-", $filename); // Replaced back [-]


			if ( strstr($filename, " + ") )
			{
				$msg = "Title can not contain + sign with spaces at both ends";
				return 0;
			}

			if ( stristr($filename, "&") && !stristr($filename, "-&-") )
			{
				$msg = "Error in title. `&` can only be used with spaces around it.<br>Please update title so that & has leading and trailing whitespaces. -validateProductName-";
				return 0;
			}

			if ( $categoryId > 0 )
			{
				$tempFilename = handleSingleQuote($filename);

				if ( $productId == 0 )
					$res = executeQuery("SELECT ID FROM ".PREFIX."PRODUCT WHERE CATEGORY_ID='".$categoryId."' AND FILENAME = '".$tempFilename."'", 0, 0);
				else
					$res = executeQuery("SELECT ID FROM ".PREFIX."PRODUCT WHERE CATEGORY_ID='".$categoryId."' AND FILENAME = '".$tempFilename."' AND ID <> ".$productId."", 0, 0);
			
				if ( $row = getRow($res) )
				{
					$msg = "Filename should be unique. It cannot match other product`s filename in a category.";
					return 0;
				}
			}

			return 1;
		}
		else
			return 1;
	}

//*************************************************************************************************

	function isCategoryHasProducts($msg, $category)
	{
		$res = executeQuery("SELECT ID FROM ".PREFIX."PRODUCT WHERE CATEGORY_ID = '".$category."'");
		if ( $row = getRow($res) )
		{
			$msg = "Cannot add categories under this parent as it already has products. -isCategoryHasProducts-";
			return 1;
		}
		else
			return 0;
	}

//*************************************************************************************************

	function updateCategoryLevels($parentId, $level)	// update level to $level of all the childs of $id
	{
		$res = executeQuery("SELECT ID FROM ".PREFIX."CATEGORY WHERE PARENT_ID = ".$parentId);

		while ( $row = getRow($res) )
		{
			executeUpdate("UPDATE ".PREFIX."CATEGORY SET LEVEL = ".$level." WHERE ID = ".$row["ID"]);

			updateCategoryLevels($row["ID"], $level+1);
		}
	}

//*************************************************************************************************

	function isCategoryHasChildCategory($msg, $category)
	{
		$res = executeQuery("SELECT ID FROM ".PREFIX."CATEGORY WHERE PARENT_ID = '".$category."'");
		if ( $row = getRow($res) )
		{
			$msg = "Product cannot be added to this category because it has sub-categories.<br>You can only add product at the last level of the category nesting.";
			return 1;
		}
		else
			return 0;
	}

//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************

	function addCategory($msg, $parent, $level, $name, $filename='', $isValid='Y', $homepageDisplay='N', $sortOrder=0)
	{
		$name				= handleSingleQuote($name);
		$filename			= handleSingleQuote($filename);

		if ( !isset($parent) )
		{
			$msg = "Category`s parent cannot be empty. Please select parent and retry. -addCategory-";
			return 0;
		}

		if ( $name == "" )
		{
			$msg = "Category name cannot be empty. Please fill the form completely and retry. -addCategory-";
			return 0;
		}
		
		if ( isCategoryHasProducts(&$msg, $parent) )
			return 0;

		if ( validateCategoryName(&$msg, &$name, &$filename) == 0 )
			return 0;

		// Level is empty OR Zero then CALCULATE it
		if ( $level == '' )
			$level = $parent==0?0:(getColumn("CATEGORY", $parent, "LEVEL", "ID")+1);
		
		$categoryId	= getNextId("CATEGORY");
		$sql		= "INSERT INTO ".PREFIX."CATEGORY (ID, PARENT_ID, LEVEL, TITLE, FILENAME, SORT_ORDER, ISVALID, HOMEPAGE_DISPLAY) VALUES('".$categoryId."', '".(int)$parent."', '".$level."', '".$name."', '".$filename."', '".$sortOrder."', '".$isValid."', '".$homepageDisplay."')";
		$res		= executeUpdate($sql);
		//echo $sql.'<br><br>';
		
		if ( $res > 0 )
		{
			$msg = "2";
			return $categoryId;
		}
		else
		{
			$msg = "6";
			return 0;
		}
	}

//*************************************************************************************************

	// $isUpdateAll is used to update all columns (except ENUM columns) even if the value is empty. 
	// It is usually called in the update where TEXT values can be reset.
	//
	function updateCategory($msg, $categoryId, $parent, $level, $name, $filename='', $isValid='', $homepageDisplay='', $sortOrder='', $isUpdateAll=0)
	{
		$name				= handleSingleQuote($name);
		$filename			= handleSingleQuote($filename);
		
		if ( $categoryId == "" )
		{
			$msg = "Empty CategoryID passed. -updateCategory-";
			return 0;
		}

		if ( $isUpdateAll == 1 )
		{
			if ( $name == "" )
			{
				$msg = "Category name cannot be empty. -updateCategory-";
				return 0;
			}
		}

		$sql = "UPDATE ".PREFIX."CATEGORY SET ";
		
		if ( isset($parent) )
		{
			
			if ( isCategoryHasProducts(&$msg, $parent) )
				return 0;

			if ( $parent == $categoryId )
			{
				$msg = "Cannot add category to itself. Please select another parent category.";
				return 0;
			}

			$sql .= "PARENT_ID = '".$parent."', ";

			// Level is empty OR Zero then CALCULATE it
			if ( $level == '' )
				$level = $parent==0?0:(getColumn("CATEGORY", $parent, "LEVEL", "ID")+1);
			$sql .= "LEVEL = '".$level."', ";

			$parentFlag = 1;
		}

		if ( $isUpdateAll == 1 || $name != "" )
			$sql .= "TITLE = '".$name."', ";

		if ( $isUpdateAll == 1 || $filename != "" )
		{
			// Filename is to be changed, validate it
			if ( validateCategoryName(&$msg, &$name, &$filename, $categoryId) == 0 )
				return 0;

			$sql .= "FILENAME = '".$filename."', ";
		}

		if ( $isUpdateAll == 1 || $sortOrder != "" )
			$sql .= "SORT_ORDER = '".$sortOrder."', ";

		if ( $isValid != "" )
		{
			$sql .= "ISVALID = '".$isValid."', ";
			$validFlag = 1;
		}

		if ( $homepageDisplay != "" )
			$sql .= "HOMEPAGE_DISPLAY = '".$homepageDisplay."', ";

		$sql .= "LAST_UPDATED = '".getTimezoneTime($TIMEZONE)."' WHERE ID = ".$categoryId;

		//echo $sql.'['.$parent.']';die;
		$res		= executeUpdate($sql);
		
		if ( $res > 0 )
		{
			if ( $parentFlag == 1 )
				updateCategoryLevels($categoryId, $level+1);

			if ( $validFlag  == 1 )
			{
				$sIds = statusRecursive("CATEGORY", "PARENT_ID", "ID", $categoryId, $isValid);
				executeUpdate("UPDATE ".PREFIX."PRODUCT SET ISVALID = '".$isValid."' WHERE CATEGORY_ID IN (".$sIds."0)");
			}

			$msg = "3";
			return 1;
		}
		else
		{
			$msg = "6";
			return 0;
		}

	}

//*************************************************************************************************

	function deleteCategory($msg, $categoryId)
	{
		if ( $categoryId == "" )
		{
			$msg = "Empty CategoryID passed. -deleteCategory-";
			return 0;
		}

		$sIds = delRecursive("CATEGORY", "PARENT_ID", "ID", $categoryId, PATH."upload/category/");
		//executeUpdate("DELETE FROM ".PREFIX."PRODUCT WHERE CATEGORY_ID IN (".$sIds."0)");

		$res  = executeQuery("SELECT ID FROM ".PREFIX."PRODUCT WHERE CATEGORY_ID IN (".$sIds."0)");
		while ( $row = getRow($res) )
			deleteProduct('', $row['ID']);

		$msg  = "4";
	}

//*************************************************************************************************

	function uploadCategoryImage($msg, $categoryId, $image)
	{
		if ( $categoryId == "" )
		{
			$msg = "Empty CategoryID passed. -updateCategory-";
			return 0;
		}

		if ( $image == "" )
		{
			$msg = "1";
			return 0;
		}

		$filename = PATH."upload/category/".$categoryId.".jpg";

		if ( @copy($image["tmp_name"], $filename) )
		{
			$arr = getimagesize($filename);

			if ( $arr !== false )
			{
				$msg = "10";
				return 1;
			}
			else
			{
				@unlink($filename);
				$msg = "Incorrect Mime type";
			}
		}		else
			$msg = "6";
	}

//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************

	function addProduct($msg, $category, $name, $filename, $detail, $manufacturer='', $manufacturerCode='', $isValid='Y', $specialOffer='N', $minPrice='0.00', $maxPrice='0.00', $storeCount=0)
	{
		$name				= handleSingleQuote($name);
		$filename			= handleSingleQuote($filename);
		$detail				= handleSingleQuote($detail);
		$manufacturer		= handleSingleQuote($manufacturer);
		$manufacturerCode	= handleSingleQuote($manufacturerCode);

		if ( $category == '' )
		{
			$msg = "Category cannot be empty. Please select category and retry. -addProduct-";
			return 0;
		}

		if ( $name == '' )
		{
			$msg = "Product name cannot be empty. Please fill the form completely and retry. -addProduct-";
			return 0;
		}

		if ( isCategoryHasChildCategory(&$msg, $category) )
			return 0;

		if ( validateProductName(&$msg, &$name, &$filename, $category) == 0 )
			return 0;

		$productId	= getNextId("PRODUCT");
		$sql		= "INSERT INTO ".PREFIX."PRODUCT(ID, CATEGORY_ID, TITLE, FILENAME, DETAIL, MANUFACTURER, MANUFACTURER_CODE, MIN_PRICE, MAX_PRICE, STORE_COUNT, ISVALID, SPECIAL_OFFER, DATE_ADDED) VALUES('".$productId."', '".$category."', '".$name."', '".$filename."', '".$detail."', '".$manufacturer."', '".$manufacturerCode."', '".$minPrice."', '".$maxPrice."', '".$storeCount."', '".$isValid."', '".$specialOffer."', '".getTimezoneTime($TIMEZONE)."')";
		$res		= executeUpdate($sql);
		//echo $sql.'<br><br>';
		
		if ( $res > 0 )
		{
			$msg = "2";

			addProductCode('', $productId, $manufacturerCode);
			return $productId;
		}
		else
		{
			$msg = "6";
			return 0;
		}
	}
		
//*************************************************************************************************

	function updateProduct($msg, $productId, $category, $name, $filename, $detail, $manufacturer='', $manufacturerCode='', $isValid='', $specialOffer='', $minPrice='', $maxPrice='', $storeCount='', $isUpdateAll=0)
	{
		$name				= handleSingleQuote($name);
		$filename			= handleSingleQuote($filename);
		$detail				= handleSingleQuote($detail);
		$manufacturer		= handleSingleQuote($manufacturer);
		$manufacturerCode	= handleSingleQuote($manufacturerCode);

		if ( $productId == "" )
		{
			$msg = "Empty ProductID passed. -updateProduct-";
			return 0;
		}

		if ( $isUpdateAll == 1 )
		{
			if ( $name == '' )
			{
				$msg = "Product name cannot be empty. Please specify product name. -updateProduct-";
				return 0;
			}

			if ( $category == '' )
			{
				$msg = "Category cannot be empty. Please select category and retry. -updateProduct-";
				return 0;
			}
		}
		
		$sql = "UPDATE ".PREFIX."PRODUCT SET ";
		
		if ( $category != '' )
		{
			if ( isCategoryHasChildCategory(&$msg, $category) )
				return 0;

			$sql .= "CATEGORY_ID = '".$category."', ";
		}

		if ( $isUpdateAll == 1 || $name != "" )
			$sql .= "TITLE = '".$name."', ";

		if ( $isUpdateAll == 1 || $filename != "" )
		{
			// Filename is to be updated, validate it
			if ( validateProductName(&$msg, &$name, &$filename, $category, $productId) == 0 )
				return 0;
			
			$sql .= "FILENAME = '".$filename."', ";
		}

		if ( $isUpdateAll == 1 || $detail != "" )
			$sql .= "DETAIL = '".$detail."', ";

		if ( $isUpdateAll == 1 || $manufacturer != "" )
			$sql .= "MANUFACTURER = '".$manufacturer."', ";

		if ( $isUpdateAll == 1 || $manufacturerCode != "" )
			$sql .= "MANUFACTURER_CODE = '".$manufacturerCode."', ";

		if ( $isValid != "" )
			$sql .= "ISVALID = '".$isValid."', ";

		if ( $specialOffer != "" )
			$sql .= "SPECIAL_OFFER = '".$specialOffer."', ";

		if ( $minPrice != "" )
			$sql .= "MIN_PRICE = '".$minPrice."', ";

		if ( $maxPrice != "" )
			$sql .= "MAX_PRICE = '".$maxPrice."', ";

		if ( $storeCount != "" )
			$sql .= "STORE_COUNT = '".$storeCount."', ";

		$sql .= "LAST_UPDATED = '".getTimezoneTime($TIMEZONE)."' WHERE ID = ".$productId;

		$res  = executeUpdate($sql);
		
		if ( $res > 0 )
		{
			// Add Product Code
			addProductCode('', $productId, $manufacturerCode);

			$msg = "3";
			return 1;
		}
		else
		{
			$msg = "6";
			return 0;
		}
	}
		
//*************************************************************************************************

	function deleteProduct($msg, $productId)
	{
		if ( $productId == "" )
		{
			$msg = "Empty ProductID passed. -deleteProduct-";
			return 0;
		}

		$res = executeUpdate("DELETE FROM ".PREFIX."PRODUCT WHERE ID = ".$productId);
		
		if ( $res > 0 )
		{
			$res1 = executeQuery("SELECT ID FROM ".PREFIX."USER_PRODUCT WHERE PRODUCT_ID = ".$productId);
			while ( $row1 = getRow($res1) )
			{
				executeQuery("DELETE FROM ".PREFIX."CLICKTHROUGHS WHERE USER_PRODUCT_ID = ".$row1["ID"]);
			}
			free($res1);
			executeQuery("DELETE FROM ".PREFIX."PRODUCT_RATING WHERE PRODUCT_ID = ".$productId);
			executeQuery("DELETE FROM ".PREFIX."USER_PRODUCT WHERE PRODUCT_ID = ".$productId);
			executeQuery("DELETE FROM ".PREFIX."PRODUCT_ALIAS WHERE PRODUCT_ID = ".$productId);

			@unlink(PATH.'upload/products/'.$productId.'.jpg');

			$msg = "4";
		}
		else
			$msg = "6";
	}
		
//*************************************************************************************************

	function uploadProductImage($msg, $productId, $sourceImage, $productImage="")
	{
		if ( $productId == "" )
		{
			$msg = "Empty ProductID passed. -uploadProductImage-";
			return 0;
		}

		if ( $sourceImage == "" )
		{
			$msg = "1";
			return 0;
		}

		// Get product image name from source image name.
		$productImageFilename = basename($sourceImage);

		if ( $productImage == "" )
			$productImgPath = PATH."upload/products/".$productImageFilename;
		else
			$productImgPath = $productImage;

		if ( @copy($sourceImage, $productImgPath) )
		{
			$arr = getimagesize($productImgPath);

			if ( $arr !== false )
			{
				// Save New Image name into the Database
				$result = executeQuery("SELECT * FROM ".PREFIX."PRODUCT_IMAGE WHERE PRODUCT_ID = ".$productId." AND FILENAME = '".$productImageFilename."'");
				if ( $row = getRow($result) )
				{
					// Dont need to insert record
				}
				else
				{
					executeUpdate("INSERT INTO ".PREFIX."PRODUCT_IMAGE(PRODUCT_ID, FILENAME) VALUES(".$productId.", '".$productImageFilename."')");
				}
								
				$msg = "10";
				return 1;
			}
			else
			{
				@unlink($productImgPath);
				$msg = "Incorrect Mime type";
			}
		}
		else
			$msg = "6";

		return 0;
	}

//*************************************************************************************************

	function getProduct($msg, $name, $categoryId=0, $isExactMatch='Y')  // ExactMatch = Y or N or S --- N => Database Product name super set of needle
	{																	// --- S => Needle super set of Database Product name
		if ( $name == "" )
		{
			$msg = "Empty Product Name passed. -getProduct-";
			return 0;
		}

		$name = handleSingleQuote($name);

		if ( $isExactMatch == 'Y' )
		{
			$sql = "SELECT ID FROM ".PREFIX."PRODUCT WHERE TITLE LIKE '".$name."'";

			if ( $categoryId > 0 )
				$sql .= " AND CATEGORY_ID = ".$categoryId;

			$res = executeQuery($sql);
			if ( $row = getRow($res) )
				return $row['ID'];

			else
				return getProductAlias(&$msg, $name, $categoryId, $isExactMatch);
		}

		else if ( $isExactMatch == 'N' )
		{
			$arrWords	= explode(" ", $name);
			$likeQuery	= "";
			foreach ( $arrWords as $word )
				$likeQuery .= "CONCAT(' ', TITLE, ' ') LIKE '% ".trim($word)." %' AND ";

			if ( $categoryId > 0 )
				$likeQuery .= "CATEGORY_ID = ".$categoryId." AND ";

			$sql = "SELECT ID FROM ".PREFIX."PRODUCT WHERE ".$likeQuery." 1 = 1";
			$res = executeQuery($sql);
			if ( $row = getRow($res) )
				return $row['ID'];

			else
				return getProductAlias(&$msg, $name, $categoryId, $isExactMatch);
		}

		else if ( $isExactMatch == 'S' )
		{
			if ( $categoryId > 0 )
				$likeQuery .= "CATEGORY_ID = ".$categoryId." ";
			else
				$likeQuery .= "1 = 1";

			$productId = matchColumnInDbWithCSV("PRODUCT", "TITLE", "ID", $name, $likeQuery);
			
			if ( $productId > 0 )
				return $productId;

			else
				return getProductAlias(&$msg, $name, $categoryId, $isExactMatch);
		}

	}

//*************************************************************************************************

	function getProductAlias($msg, $name, $categoryId=0, $isExactMatch='Y')  // ExactMatch = Y or N or S --- N => Database Product name super set of needle
	{																		 // --- S => Needle super set of Database Product name
		if ( $name == "" )
		{
			$msg = "Empty Product Name passed. -getProductAlias-";
			return 0;
		}
		
		$name = handleSingleQuote($name);

		if ( $isExactMatch == 'Y' )
		{
			$sql = "SELECT P.ID AS ID FROM ".PREFIX."PRODUCT P, ".PREFIX."PRODUCT_ALIAS PA WHERE P.ID = PA.PRODUCT_ID AND PA.TITLE LIKE '".$name."'";

			if ( $categoryId > 0 )
				$sql .= " AND P.CATEGORY_ID = ".$categoryId;

			$res = executeQuery($sql);
			if ( $row = getRow($res) )
				return $row['ID'];

			else
				return 0;
		}

		else if ( $isExactMatch == 'N' )
		{
			$arrWords	= explode(" ", $name);
			$likeQuery	= "";
			foreach ( $arrWords as $word )
				$likeQuery .= "CONCAT(' ', PA.TITLE, ' ') LIKE '% ".trim($word)." %' AND ";

			if ( $categoryId > 0 )
				$likeQuery .= "P.CATEGORY_ID = ".$categoryId." AND ";

			$sql = "SELECT P.ID AS ID FROM ".PREFIX."PRODUCT P, ".PREFIX."PRODUCT_ALIAS PA WHERE ".$likeQuery." P.ID = PA.PRODUCT_ID";
			$res = executeQuery($sql);
			if ( $row = getRow($res) )
				return $row['ID'];

			else
				return 0;
		}

		else if ( $isExactMatch == 'S' )
		{
			if ( $categoryId > 0 )
				$likeQuery .= "P.ID = PA.PRODUCT_ID AND P.CATEGORY_ID = ".$categoryId." ";
			else
				$likeQuery .= "P.ID = PA.PRODUCT_ID";

			$productId = matchColumnInDbWithCSV("PRODUCT P, ".PREFIX."PRODUCT_ALIAS PA ", "PA.TITLE", "P.ID", $name, $likeQuery);
			
			if ( $productId > 0 )
				return $productId;

			else
				return 0;
		}

	}

//*************************************************************************************************

	function addProductCode($msg, $productId, $code)
	{
		if ( $productId == "" )
		{
			$msg = "Empty Product ID passed. -addProductCode-";
			return 0;
		}

		if ( $code == "" )
		{
			$msg = "Empty Product Code passed. -addProductCode-";
			return 0;
		}

		$res = executeQuery("SELECT PRODUCT_ID FROM ".PREFIX."PRODUCT_CODE_ALIAS WHERE MANUFACTURER_CODE = '".$code."'");
		if ( $row = getRow($res) )
		{
			if ( $row['PRODUCT_ID'] == $productId )
			{
				$msg = "Manufacturer Code already exists pointing to same product [ID:$productId][Code:$code]";
				return 1;
			}
			else
			{
				$msg = "Manufacturer Code already exists but pointed to a different product. Now updating to product [ID:$productId][Code:$code]";
				
				$res = executeUpdate("UPDATE ".PREFIX."PRODUCT_CODE_ALIAS SET PRODUCT_ID = '".$productId."' WHERE MANUFACTURER_CODE = '".$code."')");
				
				if ( $res > 0 )
					return 1;

				else
					return 0;
			}
		}
		else
		{
			$msg = "Manufacturer Code does not exists. Now inserting to product [ID:$productId][Code:$code]";
			$res = executeUpdate("INSERT INTO ".PREFIX."PRODUCT_CODE_ALIAS VALUES ('".getNextId('PRODUCT_CODE_ALIAS')."', '".$productId."', '".$code."')");
			
			if ( $res > 0 )
				return 1;

			else
				return 0;
		}
	}

//*************************************************************************************************

	function getProductByCode($msg, $code, $categoryId=0)
	{
		if ( $code == "" )
		{
			$msg = "Empty Product Code passed. -getProductByCode-";
			return 0;
		}

//		if ( $categoryId > 0 )
//			$likeQuery .= "P.CATEGORY_ID = ".$categoryId." AND ";

		$sql = "SELECT PRODUCT_ID FROM ".PREFIX."PRODUCT_CODE_ALIAS WHERE MANUFACTURER_CODE LIKE '".$code."'";		
		$res = executeQuery($sql);
		if ( $row = getRow($res) )
			return $row['PRODUCT_ID'];

		else
			return 0;
	}	

//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************

	function addUserProduct($msg, $userId, $productId, $price, $comment, $merchantBrand, $detailUrl, $buyUrl, $shipping='', $merchantPartNo='', $isAvailable='Y', $isValid='Y')
	{
		if ( $userId == "" )
		{
			$msg = "Empty UserID passed. -addUserProduct-";
			return 0;
		}

		if ( $productId == "" )
		{
			$msg = "Empty ProductID passed. -addUserProduct-";
			return 0;
		}

		if ( $price == '' )
		{
			$msg = "Product price cannot be empty. Please provide price and retry. -addUserProduct-";
			return 0;
		}

		if ( $buyUrl == "" )
		{
			$msg = "Product BuyURL cannot be empty. Please provide BuyURL and retry. -addUserProduct-";
			return 0;
		}


		$comment			= handleSingleQuote($comment);
		$merchantBrand		= handleSingleQuote($merchantBrand);
		$shipping			= handleSingleQuote($shipping);
		$merchantPartNo		= handleSingleQuote($merchantPartNo);

		$price				= str_replace(',', '', $price);

		/*
		$allowedProducts	= getMerchantAllowedProducts($userId);
		$activeProducts		= getMerchantCurrentValidProducts($userId);
		$isValid			= $activeProducts >= $allowedProducts ? "N":"Y";
		*/

		$userProductId	= getNextId("USER_PRODUCT");
		$sql			= "INSERT INTO ".PREFIX."USER_PRODUCT(ID, USERID, PRODUCT_ID, PRICE, COMMENT, BRAND, DETAIL_URL, BUY_URL, SHIPPING, PARTNO, ISAVAILABLE, ISVALID, LAST_UPDATED) VALUES(".$userProductId.", '".$userId."', '".$productId."', '".$price."', '".$comment."', '".$merchantBrand."', '".$detailUrl."', '".$buyUrl."', '".$shipping."', '".$merchantPartNo."', '".$isAvailable."', '".$isValid."', '".getTimezoneTime($TIMEZONE)."')";
		$res			= executeUpdate($sql);

		if ( $res > 0 )
		{
			// Update Price Profile
			updateProductPrice($productId, $userId, $userProductId);

			$msg = "2";
			return $userProductId;
		}
		else
		{
			$msg = "6";
			return 0;
		}
	}

//*************************************************************************************************

	function updateUserProduct($msg, $userProductId, $userId, $productId, $price, $comment, $merchantBrand, $detailUrl, $buyUrl, $shipping='', $merchantPartNo='', $isAvailable='', $isValid='', $isUpdateAll=0)
	{
		if ( $userProductId == "" )
		{
			$msg = "Empty UserProductID passed. -updateUserProduct-";
			return 0;
		}

		if ( $isUpdateAll == 1 )
		{
			if ( $productId == "" || $productId == "0" )
			{
				$msg = "Empty ProductID passed. Please select a product and proceed. -updateUserProduct-";
				return 0;
			}

			if ( $price == '' )
			{
				$msg = "Product price cannot be empty. Please provide price and retry. -updateUserProduct-";
				return 0;
			}

			if ( $buyUrl == "" )
			{
				$msg = "Product BuyURL cannot be empty. Please provide BuyURL and retry. -updateUserProduct-";
				return 0;
			}
		}

		$comment			= handleSingleQuote($comment);
		$merchantBrand		= handleSingleQuote($merchantBrand);
		$shipping			= handleSingleQuote($shipping);
		$merchantPartNo		= handleSingleQuote($merchantPartNo);

		$price				= str_replace(',', '', $price);

		$sql = "UPDATE ".PREFIX."USER_PRODUCT SET ";
		
		if ( $userId != "" )
			$sql .= "USERID = '".$userId."', ";

		if ( $productId != "" )
			$sql .= "PRODUCT_ID = '".$productId."', ";

		if ( $price != '' )
			$sql .= "PRICE = '".$price."', ";

		if ( $isUpdateAll == 1 || $comment != "" )
			$sql .= "COMMENT = '".$comment."', ";

		if ( $isUpdateAll == 1 || $merchantBrand != "" )
			$sql .= "BRAND = '".$merchantBrand."', ";

		if ( $isUpdateAll == 1 || $detailUrl != "" )
			$sql .= "DETAIL_URL = '".$detailUrl."', ";

		if ( $buyUrl != "" )
			$sql .= "BUY_URL = '".$buyUrl."', ";

		if ( $isUpdateAll == 1 || $shipping != "" )
			$sql .= "SHIPPING = '".$shipping."', ";

		if ( $isUpdateAll == 1 || $merchantPartNo != "" )
			$sql .= "PARTNO = '".$merchantPartNo."', ";

		if ( $isUpdateAll == 1 || $isAvailable != "" )
			$sql .= "ISAVAILABLE = '".( stristr($isAvailable, "Y")?"Y":"N" )."', ";

		if ( $isValid != "" )
		{
			if ( $isValid == 'Y' )
			{
				/*
				if ( $userId != '' )
				{
					$allowedProducts	= getMerchantAllowedProducts($userId);
					$activeProducts		= getMerchantCurrentValidProducts($userId);
					$isValid			= $activeProducts >= $allowedProducts ? "N":"Y";
				}
				*/
				$sql .= "ISVALID = '".$isValid."', ";
			}
			else
				$sql .= "ISVALID = '".$isValid."', ";
		}


		$sql .= " LAST_UPDATED = '".getTimezoneTime($TIMEZONE)."' WHERE ID = ".$userProductId;
					
		$res  = executeUpdate($sql);
		
		if ( $res > 0 )
		{
			// Update Price Profile
			updateProductPrice($productId, $userId, $userProductId);

			$msg = "3";
			return 1;
		}
		else
		{
			$msg = "6";
			return 0;
		}
	}

//*************************************************************************************************

	function deleteUserProduct($msg, $userProductId)
	{
		if ( $userProductId == "" )
		{
			$msg = "Empty UserProductID passed. -deleteUserProduct-";
			return 0;
		}
		
		executeQuery("DELETE FROM ".PREFIX."CLICKTHROUGHS WHERE USER_PRODUCT_ID = ".$userProductId);

		// Update Price Profile
		updateProductPrice('', '', $userProductId, 1);

		$msg = "4";
		return 1;	
	}
		

//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************

	function addUser($msg, $userId, $password, $confirmPassword, $companyName, $personName, $address, $city, $state, $country, $zip, $phone, $mobile, $email, $url, $verificationCode='1', $isValid='Y', $spiderProducts='N', $spiderSearchWithin='<table', $isClickthroughFree='N', $isFeatured='N')
	{
		if ( $userId == "" )
		{
			$msg = "Empty UserID entered. Please provide userId and then retry. -addUser-";
			return 0;
		}

		if ( ereg('[^A-Za-z0-9\.\_]', $userId) )
		{
			$msg = "Invalid characters in UserID. UserID can only contain alphanumeric characters, a dot (.) or an underscore (_). Please provide another userId and then retry. -addUser-";
			return 0;
		}

		if ( $password == "" || $confirmPassword == "" )
		{
			$msg = "Empty Password or ConfirmPassword entered. Please provide it and then retry. -addUser-";
			return 0;
		}

		if ( $companyName == "" )
		{
			$msg = "Empty name entered. Please provide name and then retry. -addUser-";
			return 0;
		}

		if ( $email == "" )
		{
			$msg = "Empty email address entered. Please provide a valid email address and then retry. -addUser-";
			return 0;
		}

		if ( $url == "" )
		{
			$msg = "Empty website's link entered. Please provide a valid website's link and then retry. -addUser-";
			return 0;
		}

		if ( $password != $confirmPassword )
		{
			$msg = "Password and ConfirmPassword do not match. Please retype both passwords carefully and then retry. -addUser-";
			return 0;
		}

		if ( $userId == "admin" || $userId == "merchant" )
		{
			$msg = "You cannot register with a reserved UserID. Please retry with another userId. -addUser-";
			return 0;
		}

		if ( $row = getRecord("USER", $userId, "USERID") || $row = getRecord("USERINFO", $userId, "USERID") )
		{
			$msg = "UserID already exists in database. Please register with another userId. -addUser-";
			return 0;
		}

		if ( $row = getRecord("USERINFO", $email, "EMAIL") )
		{
			$msg = "Email address already exists, please specify another email address.<br>If you have already registered with us please use 'forgot password form' to retrive your password. -addUser-";
			return 0;
		}

		$companyName		= handleSingleQuote($companyName);
		$personName			= handleSingleQuote($personName);
		$address			= handleSingleQuote($address);

		$res = executeUpdate("INSERT INTO `".PREFIX."USER` (USERID, PASSWORD, PACKAGE_ID, REGISTERATION_DATE, VERIFICATION_CODE, ISVALID, IS_CLICKTHROUGH_FREE, ISFEATURED) VALUES( '".$userId."' , '".encrypt($password)."' , 0, '".getTimezoneTime($TIMEZONE)."', '".$verificationCode."', '".$isValid."', '".$isClickthroughFree."', '".$isFeatured."')");

		if ( $res > 0 )
		{
			$res = executeUpdate("INSERT INTO ".PREFIX."USERINFO (USERID, NAME, PERSON_NAME, ADDRESS, CITY, STATE, COUNTRY, ZIP, PHONE, MOBILE, EMAIL, URL, SPIDER_PRODUCTS, SPIDER_SEARCH_WITHIN) VALUES ('".$userId."', '".$companyName."', '".$personName."', '".$address."', '".$city."', '".$state."', '".$country."', '".$zip."', '".$phone."', '".$mobile."', '".$email."', '".$url."', '".$spiderProducts."', '".$spiderSearchWithin."')");
			
			if ( $res > 0 )
			{
				makeFolder("upload/merchant/".$userId);

				if ( file_exists(PATH.'upload/merchant/'.$userId) )
				{
					$msg = 'Registeration process completed successfully';
					return 1;
				}
				else
				{
					$msg = 'Registeration process failed. Partial record only inserted.<br>You will not be able to process datafeeds.<br>Please contact webmaster for further assistance.<br>ECode:-addUser-03';
					return 0;
				}
			}
			else
			{
				$msg = 'Registeration process failed. Partial record only inserted.<br>Please contact webmaster for further assistance.<br>ECode:-addUser-02';
				return 0;
			}
		}
		else
		{
			$msg = 'Registeration Failed. Record could not be inserted.<br>Please retry. If problem persists please contact webmaster for further assistance.<br>ECode:-addUser-01';
			return 0;
		}	
	}


//*************************************************************************************************

	function addUserPackage($msg, $userId, $packageId, $paymentType=1)
	{
		if ( $userId == "" )
		{
			$msg = "Empty UserID entered. Please provide userId and then retry. -addUserPackage-";
			return 0;
		}

		if ( $packageId == "" )
		{
			$msg = "Package not selected. Please select a package and then retry. -addUserPackage-";
			return 0;
		}

		$amount	= (int)getColumn("PACKAGE", $packageId, "PRICE");

		$id		= getNextId("PACKAGE_PAYMENT");
		$ret	= executeUpdate("INSERT INTO ".PREFIX."PACKAGE_PAYMENT (ID, MERCHANTID, PACKAGE_ID, PAYMENT_TYPE, PAYMENT_DATETIME, AMOUNT, ISVALID) VALUES( ".$id.", '".$userId."', '".$packageId."', ".$paymentType.", '".getTimezoneTime($TIMEZONE)."', '".$amount."', '".($amount==0 ? 'Y':'N')."')");

		if ( $ret > 0 )
		{
			if ( $amount == 0 )
			{
				executeUpdate("UPDATE `".PREFIX."USER` SET PACKAGE_ID = ".$packageId.", ISVALID = 'Y' WHERE USERID = '".$userId."'");
				$msg = 'Package selection successful for merchant [UserID:'.$userId.']<br>You may continue by login into merchant area.';
				return 1;
			}
			else
			{
				executeUpdate("UPDATE `".PREFIX."USER` SET PACKAGE_ID = ".$packageId.", ISVALID = 'N' WHERE USERID = '".$userId."'");
				$msg = 'Package selection successful for merchant [UserID:'.$userId.']<br>You will not be able to login into merchant area until your payment is verified by the Administrator.';
				return 1;
			}
		}
		else
		{
			$msg = 'Package selection failed for merchant [UserID:'.$userId.']<br>Please contact webmaster for further asssistance';
			return 0;
		}
	}

//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************

		function getProductRating($productId, $rating='')
		{
			require (PATH."include/constants.php");
			require (PATH."include/site_constants.php");

			$retVal = "";

			if ( (int)$productId == 0 )
			{
				return 'PrductID Empty';
			}

			if ( $rating == '' )
			{
				$res = executeQuery("SELECT ROUND(IFNULL(AVG(RATING), -1), 0) AS RATING FROM ".PREFIX."PRODUCT_RATING WHERE PRODUCT_ID = ".$productId." AND ISVALID = 'Y'");
				
				if ( $row = getRow($res) )
					$rating = $row["RATING"];
				else
					$rating = -1;
			}			

			if ( $rating < 0 )
			{
				$retVal .= "<a href='".$ROOT."product_rating.".$FILEEXT."?productId=".$productId."' class=link>".NO_RATING."</a>";
			}
			else
			{
				for ( $i=0; $i<10; $i++ )
				{
					$retVal .= "<a href='".$ROOT."product_rating.".$FILEEXT."?productId=".$productId."' class='link'><img src=\"".$ROOT."images/star_".($i<$rating?"a":"b")."_".($i%2).".jpg\" border='0'></a>";
				}
			}
			
			return $retVal;
		}

//*************************************************************************************************

		function getMerchantRating($userId, $rating='')
		{
			require (PATH."include/constants.php");
			require (PATH."include/site_constants.php");

			$retVal = "";

			if ( $userId == '' )
			{
				return 'UserID Empty';
			}

			if ( $rating == '' )
			{
				$res = executeQuery("SELECT ROUND(IFNULL(AVG(RATING), -1), 0) AS RATING FROM ".PREFIX."USER_RATING WHERE MERCHANTID = '".$userId."' AND ISVALID = 'Y'");
				
				if ( $row = getRow($res) )
					$rating = $row["RATING"];
				else
					$rating = -1;
			}			

			if ( $rating < 0 )
			{
				$retVal .= "<a href='".$ROOT."user_rating.".$FILEEXT."?userId=".$userId."' class=link>".NO_RATING."</a>";
			}
			else
			{
				for ( $i=0; $i<10; $i++ )
				{
					$retVal .= "<a href='".$ROOT."user_rating.".$FILEEXT."?userId=".$userId."' class='link'><img src=\"".$ROOT."images/star_".($i<$rating?"a":"b")."_".($i%2).".jpg\" border='0'></a>";
				}
			}
			
			return $retVal;
		}

//*************************************************************************************************

		function getProductThumbnail($pId, $pLink='', $width='90', $height='90', $border='white')
		{
			if ( $pId == '' )
				return 'Empty Product ID -getProductThumbnail-';

			if ( $pLink == '' )
				$pLink = getProductPath($pId);

			$out = "
				<table border='0' width='90' height='90' bgcolor='$border' cellspacing='0' align='center'>
					<tr>
						<td bgcolor='white' align='center' class='td'>";
							
							$productImage	= "upload/products/".$pId."_".defaultProductImageFilename($pId); // PATH."upload/products/".$pId.".jpg"

							if ( file_exists(PATH.$productImage) )
							{
								$imgInfo		= getimagesize(PATH.$productImage);
								/*$imgDim			= "";
								if ( $imgInfo[0] > $imgInfo[1] )
									$imgDim = "width='".$width."'";
								else
									$imgDim = "height='".$height."'";

								$out .= "<a href=\"".$pLink."\"><img src=\"".$ROOT."thumbnail.php?f=".urlencode($productImage)."&w=".$width."&h=".$height."\" border='0' ".$imgDim." alt='".$pTitle."'></a>";*/
								$out .= "<a href=\"".$pLink."\"><img src=\"".$ROOT.$productImage."\" border='0' width='".($imgInfo[0]>150?150:$imgInfo[0])."' height='".($imgInfo[1]>80?80:$imgInfo[1])."' alt='".$pTitle."'></a>";
							}
							else
							{
								$out .= "- no image -";
							}
			$out .= "
						</td>
					</tr>
				</table>";

			return $out;
		}

//*************************************************************************************************
//*************************************************************************************************
//*************************************************************************************************
?>